@@ -307,9 +307,9 @@ class IEmuHelper:
     of life reasons.
     """
 
-    def __init__(self, unicorn_engine, state):
+    def __init__(self, zebracorn_engine, state):
         self.state = state
-        self._uc = unicorn_engine
+        self._uc = zebracorn_engine
         self._logger = logging.getLogger(__name__)
         self._is_running = False
         self._managed_ctype_buffers = dict()
@@ -464,9 +464,11 @@ class IEmuHelper:
     def hook_add(
         self, htype, callback, user_data=None, begin=1, end=0, arg1=0
     ):
+        assert not self.is_running
         return self._uc.hook_add(htype, callback, user_data, begin, end, arg1)
 
     def hook_del(self, h):
+        assert not self.is_running
         return self._uc.hook_del(h)
 
     def mem_map(
@@ -740,8 +742,8 @@ def create_emulator(arch, mode, state) -> IEmuHelper:
     """
     Factory method for constructing the appropriate IEmuHelper
     """
-    from unicorn import Uc, UC_ARCH_ARM, UC_ARCH_MIPS, UC_ARCH_X86
-    from unicorn.unicorn import UcError
+    from zebracorn import Uc, UC_ARCH_ARM, UC_ARCH_MIPS, UC_ARCH_X86
+    from zebracorn import UcError
 
     try:
         uc = Uc(arch, mode)
@@ -768,6 +770,6 @@ def create_emulator(arch, mode, state) -> IEmuHelper:
             )
     except UcError:
         raise ZelosLoadException(
-            f"Custom unicorn does not support the arch/mode/bits"
+            f"Custom zebracorn does not support the arch/mode/bits"
             + f" {arch}/{mode}/{state.bits}"
         )
