@@ -15,18 +15,15 @@
 # <http://www.gnu.org/licenses/>.
 # ======================================================================
 
-import functools
 import logging
 import ntpath
 import os
 
 from collections import namedtuple
-from shutil import copyfile
-from tempfile import mkstemp
 from typing import Optional
 
-import unicorn
 import verboselogs
+import zebracorn
 
 from capstone import (
     CS_ARCH_ARM,
@@ -41,12 +38,13 @@ from capstone import (
     CS_MODE_MIPS32,
     Cs,
 )
-from unicorn import UcError
+from zebracorn import UcError
 
 from zelos import util
 from zelos.breakpoints import BreakpointManager, BreakState
 from zelos.config_gen import _generate_without_binary, generate_config
 from zelos.exceptions import UnsupportedBinaryError, ZelosLoadException
+from zelos.feeds import FeedManager
 from zelos.file_system import FileSystem
 from zelos.hooks import ExceptionHooks, HookManager, HookType, InterruptHooks
 from zelos.network import Network
@@ -54,6 +52,7 @@ from zelos.plugin import OSPlugins
 from zelos.processes import Processes
 from zelos.state import State
 from zelos.triggers import Triggers
+from zelos.zml import ZmlParser
 
 
 class Engine:
@@ -70,14 +69,6 @@ class Engine:
         if isinstance(config, str):
             config = generate_config(config)
         self.config = config
-        # OS plugins place OS-specific, system-wide, functionality
-        # in engine.zos
-
-        class ZOS(object):
-            def __init__(self):
-                pass
-
-        self.zos = ZOS()
 
         binary = config.filename
 
@@ -87,8 +78,6 @@ class Engine:
             [] if config.cmdline_args is None else config.cmdline_args
         )
 
-        self.random_file_name = getattr(config, "random_file_name", False)
-
         self.log_level = getattr(logging, config.log.upper(), None)
         if not isinstance(self.log_level, int):
             raise ValueError("Invalid log level: %s" % config.log)
@@ -102,6 +91,7 @@ class Engine:
         # If verbose is true, print lots of info, including every
         # instruction
         self.original_file_name = ""
+        self.target_binary_path = ""
         self.main_module_name = ""
         self.main_module = None
 
@@ -109,11 +99,15 @@ class Engine:
 
         self.timer = util.Timer()
 
+        self.zml_parser = ZmlParser(self.api)
         self.hook_manager = HookManager(self, self.api)
+        self.feeds = FeedManager(
+            self.config, self.zml_parser, self.hook_manager
+        )
         self.breakpoints = BreakpointManager(self.hook_manager)
         self.interrupt_handler = InterruptHooks(self.hook_manager, self)
         self.exception_handler = ExceptionHooks(self)
-        self.files = FileSystem(self, self.hook_manager)
+        self.files = FileSystem(self, self.hook_manager, config.sandbox)
         self.processes = Processes(
             self.hook_manager,
             self.interrupt_handler,
@@ -127,6 +121,7 @@ class Engine:
 
         if binary is not None and binary != "":
             self.load_executable(binary, entrypoint_override=config.startat)
+            self.hook_manager.on_entrypoint(self.hook_manager.setup_func_hooks)
         else:
             self._initialize_zelos()  # For testing purposes.
             # If no binary is passed, default to UNIX-style paths.
@@ -158,8 +153,6 @@ class Engine:
                     f"Incorrectly formatted input to '--mount': {m}"
                 )
                 continue
-        if config.strace is not None:
-            self.zos.syscall_manager.set_strace_file(config.strace)
 
     def __del__(self):
         try:
@@ -284,27 +277,8 @@ class Engine:
             HookType.MEMORY.WRITE, hook, name="write_trace"
         )
 
-    def _first_parse(self, module_path, random_file_name=False):
+    def _first_parse(self, module_path):
         """ Function to parse an executable """
-
-        if random_file_name:
-            self.original_file_name = module_path
-            original_file_name = module_path
-            # To ensure we don't get any issues with the size of the
-            # file name, we copy the file and rename it 'target'
-            fd, temp_path = mkstemp(dir=".", suffix=".xex")
-            os.close(fd)
-            temp_filename = os.path.basename(temp_path)
-            copyfile(module_path, temp_filename)
-            module_path = temp_filename
-            self.hook_manager.register_close_hook(
-                functools.partial(os.remove, temp_filename)
-            )
-            self.logger.debug(
-                f"Setting random file name for "
-                f"{original_file_name} : {module_path}"
-            )
-
         self.logger.verbose("Parse Main Module")
 
         with open(module_path, "rb") as f:
@@ -333,12 +307,12 @@ class Engine:
         emulation
         """
 
+        self.target_binary_path = module_path
+
         original_file_name = os.path.basename(module_path)
         self.original_file_name = original_file_name
 
-        file = self._first_parse(
-            module_path, random_file_name=self.random_file_name
-        )
+        file = self._first_parse(module_path)
 
         module_path = file.Filepath
         self.main_module = file
@@ -353,7 +327,11 @@ class Engine:
 
         # We need to create this file in the file system, so that other
         # files can access it.
-        self.files.create_file(self.files.zelos_file_prefix + module_path)
+        self.files.create_file(
+            self.files.emulated_path_module.join(
+                self.files.zelos_file_prefix, module_path
+            )
+        )
 
     def _initialize_zelos(self, binary=None):
         self.state = State(self, binary, self.date)
@@ -394,7 +372,7 @@ class Engine:
         p.virtual_filename = self.config.virtual_filename
         p.virtual_path = self.config.virtual_path
 
-        if hasattr(unicorn.unicorn, "WITH_ZEROPOINT_PATCH"):
+        if hasattr(zebracorn.unicorn, "WITH_ZEROPOINT_PATCH"):
 
             def process_switch_wrapper(*args, **kwargs):
                 # Block count interrupt. Fires every 2^N blocks executed
@@ -446,20 +424,20 @@ class Engine:
 
     def step(self, count: int = 1) -> None:
         """ Steps one assembly level instruction """
-        # You might be tempted to use unicorn's "count" argument to
+        # You might be tempted to use zebracorn's "count" argument to
         # step. However, printing instruction comments relies on an
         # ad-hoc "post instruction" method.
         #
-        # Using unicorn's emu_start count argument
+        # Using zebracorn's emu_start count argument
         #   run INST hook
         #   run instruction
-        #   unicorn stops
+        #   zebracorn stops
         #
         # Current method:
         #   run INST hook (don't print)
         #   run instruction
         #   run INST hook (do print) then stop before next instruction
-        #   unicorn stops
+        #   zebracorn stops
         #
         # Of course, we can simplify when we get a post instruction
         # hook working properly.
@@ -565,7 +543,7 @@ class Engine:
                     break
                 self.processes.swap_with_next_thread()
 
-        return self.zos.syscall_manager.generate_break_state()
+        return self.kernel.generate_break_state()
 
     def _run(self, p):
         t = p.current_thread
@@ -576,7 +554,7 @@ class Engine:
         self.breakpoints._disable_breakpoints_on_start(t.getIP())
         if t.emu.is_running:
             self.logger.critical(
-                "Trying to run unicorn while unicorn is already running. "
+                "Trying to run zebracorn while zebracorn is already running. "
                 "You are entering untested waters"
             )
 
@@ -592,7 +570,7 @@ class Engine:
 
     def _should_continue(self):
         """
-        Takes the reasons for ending unicorn execution, and decides
+        Takes the reasons for ending zebracorn execution, and decides
         whether to continue or end execution
         """
 
