@@ -22,15 +22,30 @@ import capstone.x86_const as cs_x86
 
 from termcolor import colored
 
-from zelos import HookType, IPlugin
+from zelos import CommandLineOption, IPlugin
 from zelos.exceptions import MemoryReadUnmapped
 
 
-class Comment:
-    def __init__(self, address, thread_id, text):
-        self.address = address
-        self.thread_id = thread_id
-        self.text = text
+CommandLineOption(
+    "trace_off",
+    action="store_true",
+    help="Turns off printing on the command line",
+)
+
+CommandLineOption(
+    "trace_file",
+    type=str,
+    default=None,
+    help="Writes the trace to a file instead of the command line.",
+)
+
+CommandLineOption(
+    "fasttrace",
+    action="count",
+    default=0,
+    help="Enable instruction-level tracing only the first time a memory "
+    "address is reached.",
+)
 
 
 class Trace(IPlugin):
@@ -39,45 +54,64 @@ class Trace(IPlugin):
 
         self.logger = logging.getLogger(__name__)
 
+        self.trace_file = None
+        if z.config.trace_file is not None:
+            self.trace_file = open(z.config.trace_file, "w")
+
         self.current_return_address = 0
         self.current_function_name = "???"
         self.current_api_module = "???"
 
-        # Comments (to be dumped)
-        # We should save a lot of space by moving this to a grouping by
-        # thread, rather than keeping the thread with each comment.
-        # This will break compatibility with systems like Doppler.
-        self.comments = []
         self.MAX_INDENTS = 40
         self.threads_to_print = set()
-        if z.config.tracethread != "":
-            self.threads_to_print.add(z.config.tracethread)
 
-        self.verbosity = z.config.verbosity
-        self.verbose = False
-        self.set_verbose(self.verbosity > 0)
         self.fasttrace = True if z.config.fasttrace > 0 else False
-        self.trace_on = ""
-        self.trace_off = ""
 
         self.last_instruction = None
         self.last_instruction_size = None
         self.should_print_last_instruction = False
 
-        if self.verbose:
-            self.set_hook_granularity(HookType.EXEC.INST)
+        self._inst_feed_handle = None
+        self._syscall_feed_handle = None
+
+        self._traceoff = z.config.trace_off
+        if not self._traceoff:
+            self.trace_on()
 
         if self.state.arch in ["x86", "x86_64"]:
-            self.comment_generator = x86CommentGenerator(
-                self.zelos, self.modules
-            )
+            self.comment_generator = x86CommentGenerator(z, self.modules)
         elif self.state.arch == "arm":
-            self.comment_generator = ArmCommentGenerator(
-                self.zelos, self.modules
-            )
+            self.comment_generator = ArmCommentGenerator(z, self.modules)
         else:
             self.comment_generator = EmptyCommentGenerator()
 
+        self._cmt_callbacks = []
+
+    def trace_on(self):
+        feeds = self.zelos.internal_engine.feeds
+        if self._inst_feed_handle is None:
+            self._inst_feed_handle = feeds.subscribe_to_inst_feed(
+                self.trace_insts
+            )
+        if self._syscall_feed_handle is None:
+            self._syscall_feed_handle = feeds.subscribe_to_syscall_feed(
+                self.trace_syscalls
+            )
+        self._traceoff = False
+
+    def trace_off(self):
+        feeds = self.zelos.internal_engine.feeds
+        if (
+            self._inst_feed_handle is not None
+            and len(self._cmt_callbacks) == 0
+        ):
+            feeds.unsubscribe_from_feed(self._inst_feed_handle)
+            self._inst_feed_handle = None
+        if self._syscall_feed_handle is not None:
+            feeds.unsubscribe_from_feed(self._syscall_feed_handle)
+            self._syscall_feed_handle = None
+        self._traceoff = True
+
     @property
     def cs(self):
         return self.zelos.internal_engine.cs
@@ -94,168 +128,59 @@ class Trace(IPlugin):
     def functions_called(self):
         return self.comment_generator.functions_called
 
-    @property
-    def strace_file(self):
-        return self.zelos.internal_engine.zos.syscall_manager.strace_file
-
-    @property
-    def strace(self):
-        return self.zelos.config.strace
-
-    def set_hook_granularity(self, granularity: HookType.EXEC):
-        """
-        Sets the code hook granularity to be either every instruction
-        or every block.
-        """
-        try:
-            self.zelos.delete_hook(self.code_hook_info)
-        except AttributeError:
-            pass  # first time setting code_hook_info
-
-        self.code_hook_info = self.zelos.hook_execution(
-            granularity, self.hook_code, name="code_hook"
-        )
-
-    def _check_timeout(self):
-        """
-        Check if specified timeout has elapsed and stop execution.
-        """
-        if self.zelos.internal_engine.timer.is_timed_out():
-            self.zelos.stop("timeout")
-
-    def hook_code(self, zelos, address, size):
-        """
-        Hook that is executed for each instruction or block.
-        """
-        try:
-            self._hook_code_impl(zelos, address, size)
-            self._check_timeout()
-        except Exception:
-            if self.zelos.thread is not None:
-                self.zelos.process.threads.kill_thread(self.zelos.thread.id)
-            self.logger.exception("Stopping execution due to exception")
-
-    def _hook_code_impl(self, zelos, address, size):
+    def trace_insts(self, zelos, address, size):
         # TCG Dump example usage:
         # self.emu.get_tcg(0, 0)
-        if self.zelos.thread is None:
-            self.zelos.stop("hook_code_null_thread")
-            return
-
-        self.zelos.thread.total_blocks_executed += 1
-        rev_modules = self.modules.reverse_module_functions
-        if (
-            self.zelos.thread.total_blocks_executed % 1000 == 0
-            and address not in rev_modules
+        if self.should_print_last_instruction:
+            self.bb(
+                self.last_instruction,
+                self.last_instruction_size,
+                full_trace=False,
+            )
+        self.should_print_last_instruction = True
+        if self.fasttrace and self.zelos.process.threads.block_seen_before(
+            address
         ):
-            self.zelos.swap_thread("process swap")
-            return
-
-        if self.verbose:
-            if self.should_print_last_instruction:
-                self.bb(
-                    self.last_instruction,
-                    self.last_instruction_size,
-                    full_trace=False,
-                )
-            self.should_print_last_instruction = True
-            if (
-                self.fasttrace
-                and self.zelos.process.threads.block_seen_before(address)
-            ):
-                self.should_print_last_instruction = False
+            self.should_print_last_instruction = False
 
         self.zelos.process.threads.record_block(address)
 
         self.last_instruction = address
         self.last_instruction_size = size
 
-    def traceoff(self, addr=None):
+    def trace_syscalls(self, zelos, syscall_name, args, retval):
         """
-        Disable verbose tracing. Optionally specify an address at which
-        verbose tracing is disabled.
+        Prints information regarding a syscall for the strace.
+        Note, this may not immediately print the syscall (may need to
+        wait for return value
         """
-        if addr is None:
-            self.set_verbose(False)
-        else:
-
-            def hook_traceoff(zelos, address, size):
-                self.set_verbose(False)
-
-            self.zelos.hook_execution(
-                HookType.EXEC.INST,
-                hook_traceoff,
-                name="traceoff_hook",
-                ip_low=addr,
-                ip_high=addr,
-                end_condition=lambda: True,
-            )
-
-    def traceoff_syscall(self, syscall_name):
-        """
-        Disable verbose tracing after a specific system call has
-        executed.
-        """
-
-        def hook_traceoff(zelos, sysname, args, retval):
-            if sysname == syscall_name:
-                zelos.plugins.trace.traceoff()
-
-        self.zelos.hook_syscalls(HookType.SYSCALL.AFTER, hook_traceoff)
-
-    def traceon(self, addr=None):
-        """
-        Enable verbose tracing. Optionally specify an address at which
-        verbose tracing is enabled.
-        """
-        if addr is None:
-            self.set_verbose(True)
-        else:
-
-            def hook_traceon(zelos, address, size):
-                self.set_verbose(True)
-
-            self.zelos.hook_execution(
-                HookType.EXEC.INST,
-                hook_traceon,
-                name="traceon_hook",
-                ip_low=addr,
-                ip_high=addr,
-                end_condition=lambda: True,
-            )
-
-    def traceon_syscall(self, syscall_name):
-        """
-        Enable verbose tracing after a specific system call has
-        executed.
-        """
-
-        def hook_traceon(zelos, sysname, args, retval):
-            if sysname == syscall_name:
-                self.traceon()
-
-        self.zelos.hook_syscalls(HookType.SYSCALL.AFTER, hook_traceon)
-
-    def set_verbose(self, should_set_verbose) -> None:
-        """
-        Used to set the verbosity level, and change the hooks.
-        This prevents two types of issues:
-
-        1) Running block hooks when printing individual instructions
-               This will cause the annotations that are printed to be
-               the values at the end of the block's execution
-        2) Running instruction hooks when not printing instructions
-               This will slow down the emulation (sometimes
-               considerably)
-        """
-        if self.verbose == should_set_verbose:
+        thread = zelos.thread
+        if not self.should_print_thread(thread):
+            return
+        if not self.zelos.internal_engine.kernel.should_print_syscalls:
             return
-        self.verbose = should_set_verbose
 
-        if should_set_verbose:
-            self.set_hook_granularity(HookType.EXEC.INST)
+        retstr = "void" if retval is None else f"{retval:x}"
+
+        if args is None:
+            self.zelos.logger.warning("Syscall did not call get_args")
+
+        if self.trace_file is None:
+            s = (
+                colored(f"[{thread.name}]", "magenta")
+                + " "
+                + colored(f"[SYSCALL]", "red")
+                + " "
+                + colored(f"{syscall_name}", "white", attrs=["bold"])
+                + f" ( {args} ) -> {retstr}"
+            )
+            print(s)
         else:
-            self.set_hook_granularity(HookType.EXEC.BLOCK)
+            s = (
+                f"[{thread.name}] "
+                f"[SYSCALL] {syscall_name} ( {args} ) -> {retstr}"
+            )
+            print(s, file=self.trace_file, flush=True)
 
     def bb(self, address=None, size=20, full_trace=False):
         if not self.should_print_thread():
@@ -315,13 +240,13 @@ class Trace(IPlugin):
         if addr_str is None:
             addr_str = f"{self.zelos.regs.getIP():08x}"
 
-        if self.strace:
+        if self.trace_file is not None:
             thread_str = f"[{thread}]"
             category_str = f"[{category}]"
             addr_str_str = f"[{addr_str}]"
             print(
                 f"{thread_str} {category_str} {addr_str_str} {s}",
-                file=self.strace_file,
+                file=self.trace_file,
             )
         else:
             thread_str = colored(f"[{thread}]", "magenta")
@@ -379,7 +304,7 @@ class Trace(IPlugin):
         if indent_count == -1:
             indent_count = 0
         args = "".join([i if ord(i) < 128 else "." for i in args])
-        if self.strace:
+        if self.trace_file is not None:
             s = "  " * min(indent_count, self.MAX_INDENTS) + args
         else:
             s = "  " * min(indent_count, self.MAX_INDENTS) + colored(
@@ -391,21 +316,27 @@ class Trace(IPlugin):
         """ Prints the thread, address and instruction string """
         if not self.should_print_thread():
             return
+        cmt = self._get_comment(insn)
+        for cb in self._cmt_callbacks:
+            cb(self.zelos, insn.address, cmt)
+
+        if self._traceoff:
+            return
+        ins_string = self._get_insn_string(insn, cmt)
         sep = ""
         if insn.address == self.zelos.regs.getIP():
             sep = "*"
-        address = insn.address
-        ins_string = self._get_insn_string(insn)
-        if address in self.zelos.main_binary.exported_functions:
-            function_name = self.zelos.main_binary.exported_functions[address]
-            if self.strace:
-                s = f"<{function_name}>"
+        addr = insn.address
+        if addr in self.zelos.main_binary.exported_functions:
+            fn_name = self.zelos.main_binary.exported_functions[addr]
+            if self.trace_file is not None:
+                s = f"<{fn_name}>"
             else:
-                s = colored(f"<{function_name}>", "white", attrs=["bold"])
-            self.print("INS", s, addr_str=f"{sep}{address:08x}")
-        self.print("INS", ins_string, addr_str=f"{sep}{address:08x}")
+                s = colored(f"<{fn_name}>", "white", attrs=["bold"])
+            self.print("INS", s, addr_str=f"{sep}{addr:08x}")
+        self.print("INS", ins_string, addr_str=f"{sep}{addr:08x}")
 
-    def should_print_thread(self, t=None):
+    def should_print_thread(self, t=None) -> bool:
         """
         Decides whether log statements should be printed for the given
         thread
@@ -425,42 +356,41 @@ class Trace(IPlugin):
             return True
         return t.name in self.threads_to_print
 
-    def _get_insn_string(self, insn):
+    def _get_insn_string(self, insn, cmt):
         """ Gets the string to be printed for an instruction."""
-        cmt = ""
+        result = f"{insn.mnemonic:8s} {insn.op_str}"
+        if len(cmt) > 0:
+            if self.trace_file is None:
+                cmt = colored("; " + cmt, "grey", attrs=["bold"])
+            else:
+                cmt = "; " + cmt
+            result = f"{result:60s} {cmt}"
+        return result
+
+    def _get_comment(self, insn) -> str:
         try:
-            cmt = self.comment_generator.get_comment(insn)
+            return self.comment_generator.get_comment(insn)
         except Exception as e:
             self.logger.notice(
                 f"Issue printing {insn.mnemonic} instruction comment: {e}"
             )
+        return ""
 
-        result = ""
-        insn_str = "{0}\t{1}".format(insn.mnemonic, insn.op_str)
-        if len(cmt) > 0:
-            padding = ""
-            padSize = 60 - len(insn_str)
-            if padSize < 0:
-                padSize = 1
-            for y in range(0, padSize):
-                padding += " "
-            if self.strace:
-                result += insn_str + " " + padding + " ; " + cmt
-            else:
-                result += (
-                    insn_str
-                    + " "
-                    + padding
-                    + colored(" ; " + cmt, "grey", attrs=["bold"])
-                )
-            # Log comment for the dump
-            self.comments.append(
-                Comment(insn.address, self.zelos.thread.id, cmt)
+    def hook_comments(self, callback) -> None:
+        """
+        Registers a callback that is called when
+        comments are generated for an instruction.
+
+        Args:
+            callback: Called when a comment is generated. Takes a single
+                argument. The return value of `callback` is ignored
+        """
+        self._cmt_callbacks.append(callback)
+        if self._inst_feed_handle is None:
+            feeds = self.zelos.internal_engine.feeds
+            self._inst_feed_handle = feeds.subscribe_to_inst_feed(
+                self.trace_insts
             )
-        else:
-            result += insn_str
-
-        return result
 
 
 class EmptyCommentGenerator:
@@ -513,12 +443,12 @@ class ArmCommentGenerator:
         """
         Returns the syscall name
         """
-        sm = self.zelos.internal_engine.zos.syscall_manager
+        k = self.zelos.internal_engine.kernel
         if insn.insn_name() == "svc" and insn.operands[0].imm != 0:
             syscall_num = insn.operands[0].imm - 0x900000
         else:
-            syscall_num = sm.get_syscall_number()
-        syscall_name = sm.find_syscall_name_by_number(syscall_num)
+            syscall_num = k.get_syscall_number()
+        syscall_name = k.find_syscall_name_by_number(syscall_num)
         return f"{syscall_name}"
 
     def _dst_comment(self, insn):
@@ -539,8 +469,10 @@ class ArmCommentGenerator:
         """
         dst_val = self._get_reg_or_mem_val(insn, insn.operands[0])
         src_val = self._get_reg_or_mem_val(insn, insn.operands[1], is_dst=True)
-        return f"{insn.reg_name(insn.operands[0].value.reg)} = "
-        f"load(0x{src_val:x}) = 0x{dst_val:x}"
+        return (
+            f"{insn.reg_name(insn.operands[0].value.reg)} = "
+            f"load(0x{src_val:x}) = 0x{dst_val:x}"
+        )
 
     def _str_comment(self, insn):
         """
@@ -606,14 +538,11 @@ class x86CommentGenerator:
         Returns a string representing the data pointed to by 'ptr' if
         'ptr' is a valid pointer. Otherwise, reutrns an empty string.
         """
-        try:
-            s = self.zelos.memory.read_string(ptr, 8)
+        s = self.zelos.memory._memory.try_read_string(ptr, 8)
+        if s is not None and len(s) > 2:
             # Require a certain amount of valid characters to reduce
             # false positives for string identification.
-            if len(s) > 2:
-                return f' -> "{s}"'
-        except MemoryReadUnmapped:
-            pass  # Just checking whether the string exists.
+            return f' -> "{s}"'
 
         try:
             pointer_data = self.zelos.memory.read_int(ptr)
