@@ -19,6 +19,7 @@ from __future__ import print_function
 import io
 import logging
 import os
+import sys
 
 from collections import defaultdict
 from typing import List, Optional, Tuple
@@ -83,10 +84,12 @@ class FileHandle(Handle):
         access=0,
         is_dir=False,
         file=None,
+        close_on_cleanup=True,
     ):
         super().__init__(name, parent_thread, access)
         self._file_system = file_system
         self._file = None
+        self._close_on_cleanup = close_on_cleanup
         self.is_dir = is_dir
         if is_dir:
             return
@@ -163,7 +166,7 @@ class FileHandle(Handle):
         return self._file.read(size)
 
     def cleanup(self) -> None:
-        if self._file is not None:
+        if self._file is not None and self._close_on_cleanup:
             self._file.close()
 
 
@@ -316,7 +319,32 @@ class Handles:
 
         def init_handles(p):
             # Add some default system handles
-            self.add_handle(StdIn(), handle_num=0, pid=p.pid)
+            try:
+                # Don't passthrough if
+                #  * stdin is terminal
+                #  * not TextIoWrapper (tests replace stdin)
+                passthrough_stdin = not sys.stdin.isatty() and isinstance(
+                    sys.stdin, io.TextIOWrapper
+                )
+            except ValueError:  # stdin was closed
+                self.logger.notice(
+                    "stdin was closed, not redirecting to emulated stdin"
+                )
+                passthrough_stdin = False
+
+            # Passthrough allows the emulated process to read stdin
+            # passed to Zelos
+            if passthrough_stdin:
+                self.logger.info("Passing stdin to emulated program")
+                self.new_file(
+                    "stdin_redirect",
+                    handle_num=0,
+                    file=sys.stdin.buffer,
+                    pid=p.pid,
+                    close_on_cleanup=False,
+                )
+            else:
+                self.add_handle(StdIn(), handle_num=0, pid=p.pid)
             self.add_handle(StdOut(), handle_num=1, pid=p.pid)
             self.add_handle(StdErr(), handle_num=2, pid=p.pid)
 
@@ -358,8 +386,11 @@ class Handles:
     # ALPC Port,Semaphore,WindowStation,etc.} handle
 
     def _current_thread_name(self) -> str:
-        curr_thread = self.processes.current_process.current_thread
-        return "none" if curr_thread is None else curr_thread.name
+        try:
+            curr_thread = self.processes.current_process.current_thread
+            return curr_thread.name
+        except AttributeError:
+            return "none"
 
     def new(self, T, name, access=0, handle_num=None):
         """
@@ -372,13 +403,26 @@ class Handles:
         return handle_num
 
     def new_file(
-        self, name, access=0, handle_num=None, is_dir=False, file=None
+        self,
+        name,
+        access=0,
+        handle_num=None,
+        is_dir=False,
+        file=None,
+        pid=None,
+        close_on_cleanup=True,
     ):
         parent_thread = self._current_thread_name()
         handle = FileHandle(
-            name, self.file_system, parent_thread, access, is_dir, file=file
+            name,
+            self.file_system,
+            parent_thread,
+            access,
+            is_dir,
+            file=file,
+            close_on_cleanup=close_on_cleanup,
         )
-        handle_num = self.add_handle(handle, handle_num=handle_num)
+        handle_num = self.add_handle(handle, handle_num=handle_num, pid=pid)
         return handle_num
 
     def new_socket(self, name, socket, access=0, handle_num=None):
