@@ -31,7 +31,8 @@ import zelos.util as util
 
 from zelos.emulator.base import MemoryRegion
 from zelos.enums import ProtType
-from zelos.exceptions import OutOfMemoryException
+from zelos.exceptions import MemoryReadUnmapped, OutOfMemoryException
+from zelos.hooks import HookType
 
 
 class Memory:
@@ -46,8 +47,11 @@ class Memory:
     MAX_UINT64 = 0xFFFFFFFFFFFFFFFF
     MAX_UINT32 = 0xFFFFFFFF
 
-    def __init__(self, emu, state, disableNX: bool = False) -> None:
+    def __init__(
+        self, emu, hook_manager, state, disableNX: bool = False
+    ) -> None:
         self.emu = emu
+        self._hook_manager = hook_manager
         self.state = state
         self.logger = logging.getLogger(__name__)
         self.disableNX = disableNX
@@ -56,7 +60,7 @@ class Memory:
         self.clear()
         self.heap = Heap(self, self.HEAP_BASE, self.HEAP_MAX_SIZE)
 
-        from unicorn import UC_HOOK_MEM_READ_PROT
+        from zebracorn import UC_HOOK_MEM_READ_PROT
 
         self.emu.hook_add(UC_HOOK_MEM_READ_PROT, self._hook_read_prot)
         self.mem_hooks = dict()
@@ -118,7 +122,11 @@ class Memory:
         Returns:
             Bytes corresponding to data held in memory.
         """
-        return self.emu.mem_read(addr, size)
+        val = self.emu.mem_read(addr, size)
+        hooks = self._hook_manager._get_hooks(HookType.MEMORY.INTERNAL_READ)
+        for hook in hooks:
+            hook(0, addr, size, val)
+        return val
 
     def write(self, addr: int, data: bytes) -> int:
         """
@@ -132,6 +140,9 @@ class Memory:
             Number of bytes written.
         """
         self.emu.mem_write(addr, data)
+        hooks = self._hook_manager._get_hooks(HookType.MEMORY.INTERNAL_WRITE)
+        for hook in hooks:
+            hook(0, addr, len(data), data)
         return len(data)
 
     def read_int(self, addr: int, sz: int = None, signed: bool = False) -> int:
@@ -185,26 +196,34 @@ class Memory:
         Returns:
             String read from memory.
         """
+        try:
+            return self._read_string(addr, size)
+        except Exception as e:
+            self.logger.debug(f"Couldn't read str at 0x{addr:x}: {e}")
+            return ""
+
+    def try_read_string(self, addr: int, size: int = 1024) -> Optional[str]:
+        """
+        Similar to read_string, however doesn't log on failure. Use for
+        when you are checking if a valid string exists.
+        """
+        try:
+            return self._read_string(addr, size)
+        except (MemoryReadUnmapped, UnicodeDecodeError):
+            pass
+        return None
+
+    def _read_string(self, addr: int, size: int = 1024) -> Optional[str]:
         if addr == 0:
             return ""
         data = b""
-        try:
-            for i in range(size):
-                byte = bytes(self.read(addr + i, 1))
-                if byte == b"\x00":
-                    break
-                data += byte
-            # TODO: Allow for different decodings.
-            return data.decode()
-        except Exception as e:
-            # TODO: We need to differentiate between attempts to check
-            # if a string exists or if expecting a string to exist. We
-            # shouldn't log an error if we aren't expecting the string
-            # to exist.
-            self.logger.debug(
-                "Couldn't read str at 0x{0:x}: {1}".format(addr + i, e)
-            )
-            return ""
+        for i in range(size):
+            byte = bytes(self.read(addr + i, 1))
+            if byte == b"\x00":
+                break
+            data += byte
+        # TODO: Allow for different decodings.
+        return data.decode()
 
     def read_wstring(self, addr: int, size: int = 1024) -> str:
         """
@@ -521,6 +540,13 @@ class Memory:
             shared=shared,
             reserve=reserve,
         )
+        hooks = self._hook_manager._get_hooks(HookType.MEMORY.INTERNAL_MAP)
+
+        # We chose not to pass the data for the map in the hook because
+        # it drastically reduces performances.
+        # (some ltp tests went from 2.6 -> 3.6 seconds)
+        for hook in hooks:
+            hook(0, address, size, None)
 
     def map_file(
         self,
@@ -553,6 +579,14 @@ class Memory:
             prot=prot,
             shared=shared,
         )
+        mr = self.get_region(address)
+        hooks = self._hook_manager._get_hooks(HookType.MEMORY.INTERNAL_MAP)
+
+        # We chose not to pass the data for the map in the hook because
+        # it drastically reduces performances.
+        # (some ltp tests went from 2.6 -> 3.6 seconds)
+        for hook in hooks:
+            hook(0, address, mr.size, None)
 
     def protect(self, address: int, size: int, prot: int) -> None:
         """
@@ -629,6 +663,16 @@ class Memory:
             return None
         return region.address
 
+    def get_module_base(self, name: str) -> Optional[int]:
+        regions = [
+            mr
+            for mr in self.get_regions()
+            if os.path.basename(mr.module_name) == os.path.basename(name)
+        ]
+        if len(regions) == 0:
+            return None
+        return min([mr.address for mr in regions])
+
     def get_perms(self, address: int) -> int:
         """
         Returns the permissions of the section containg the given
@@ -981,7 +1025,7 @@ class Heap:
         self.current_offset -= size
         return self.current_offset
 
-    def alloc(self, size: int, name: str = None, align: int = 0x4) -> int:
+    def alloc(self, size: int, name: str = None, align: int = 0x10) -> int:
         """
         Allocates memory to the heap. These are rounded up to the size
         of the alignment.
